/* Copyright (c) 2013 Oberon microsystems, Inc. (Switzerland)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. */

// Originally developed for the book
//   "Getting Started with the Internet of Things", by Cuno Pfister.
//   Copyright 2011 Cuno Pfister, Inc., 978-1-4493-9357-1.
//
// Version 1.1, for the .NET Micro Framework release 4.2.
//
// Internal facility. Not further documented.

using System;
using System.Diagnostics.Contracts;
using Microsoft.SPOT;

namespace Gsiot.Server
{
    public class ServerDiagnostics
    {
        public TimeSpan ServiceUptime { get; set; }
        public int AcceptErrors { get; set; }
        public int AcceptFailures { get; set; }
        public int RequestsTotal { get; set; }
        public int RequestHandlerErrors { get; set; }
        public int RequestHandlerFailures { get; set; }
        public int AvailableHeapSpace { get; set; }
    }

    public class DiagnosticResource
    {
        public bool IsOpen { get; private set; }
        public ServerDiagnostics Diagnostics { get; set; }

        public DiagnosticResource()
        {
            IsOpen = false;
        }

        public void Open()
        {
            Contract.Requires(!IsOpen);
            IsOpen = true;
        }

        public void HandleRequest(RequestHandlerContext context)
        {
            Contract.Requires(context != null);
            if (!IsOpen) { Open(); }

            context.ResponseMaxAge = 0;         // no caching (default is -1, i.e., no cache header)
            var s = "";
            // Service Uptime
            TimeSpan delta = Diagnostics.ServiceUptime;
            s = s + "Service Uptime\t" + delta + "\r\n";
            // accept errors (e.g., timeouts)
            s = s + "Accept Errors\t" + Diagnostics.AcceptErrors + "\r\n";
            // accept failures (unknown exceptions)
            s = s + "Accept Failures\t" + Diagnostics.AcceptFailures + "\r\n";
            // requests total
            s = s + "Requests Total\t" + Diagnostics.RequestsTotal + "\r\n";
            // request handler errors (e.g., timeouts)
            s = s + "Request Handler Errors\t" + Diagnostics.RequestHandlerErrors + "\r\n";
            // request handler failures (unknown exceptions)
            s = s + "Request Handler Failures\t" + Diagnostics.RequestHandlerFailures + "\r\n";
            // free memory
            s = s + "Available Heap Space\t" + Debug.GC(true) + "\r\n";
            context.SetResponse(s, "text/plain");
        }
    }
}
