using System.Collections;
using System.Diagnostics.Contracts;

namespace System.Threading
{
    static class BigLock
    {
        internal readonly static object Lock = new object();
    }

    internal sealed class BoundedQueue
    {
        AutoResetEvent nonEmptySignal = new AutoResetEvent(false);
        AutoResetEvent nonFullSignal = new AutoResetEvent(false);

        int capacity = 0;           // where (capacity >= 0)
        Queue queue = new Queue();  // where (queue != null)

        internal int Capacity         // once set, remains immutable
        {
            get { return capacity; }

            set
            {
                Contract.Requires(value > 0);
                capacity = value;
            }
        }

        internal void Enqueue(object o)
        {
            Contract.Requires(capacity > 0);
            // this method may block
            int count;
            lock (BigLock.Lock)
            {
                count = queue.Count;
                while (count == capacity)
                {
                    Monitor.Exit(BigLock.Lock);
                    nonFullSignal.WaitOne();
                    Monitor.Enter(BigLock.Lock);
                    count = queue.Count;
                }
                queue.Enqueue(o);
            }
            if (count == 0)
            {
                nonEmptySignal.Set();
            }
        }


        internal object Dequeue()
        {
            Contract.Requires(capacity > 0);
            // this method may block
            object o = null;
            int count;
            lock (BigLock.Lock)
            {
                count = queue.Count;
                while (count == 0)
                {
                    Monitor.Exit(BigLock.Lock);
                    nonEmptySignal.WaitOne();
                    Monitor.Enter(BigLock.Lock);
                    count = queue.Count;
                }
                o = queue.Dequeue();
            }
            if (count == capacity)
            {
                nonFullSignal.Set();
            }
            return o;
        }
    }
}
